require 'salt/formula'
require 'salt/matrix/gitrepo'
require 'forwardable'

class Salt::Formula::Git < Salt::Formula::Base

  Salt::Formula.register(self)

  def self.implement?(name, args)
    args.is_a? Hash and (args.has_key?(:git) or args.has_key?(:github))
  rescue
    false
  end

  # @!attribute [r] repo
  #   @api private
  #   @return [Salt::Matrix::Gitrepo]
  attr_reader :repo

  def initialize(name, dirname, args)
    super
    parse_options(@args)
    @name = name
    @repo = Salt::Matrix::Gitrepo.new(@remote, @name, @ref, dirname)
  end

  def version
    @ref
  end

  def properties
    {
      :expected => @ref,
      :actual   => (@repo.head || "(unresolvable)"),
      :type     => :git,
    }
  end

  extend Forwardable

  def_delegators :@repo, :sync, :status

  private

  def parse_options(options)
    @remote = options.delete(:git) || options.delete(:github)

    if options[:branch]
      @ref = "origin/#{options.delete(:branch)}"
    end

    if options[:tag]
      @ref = options.delete(:tag)
    end

    if options[:commit]
      @ref = options.delete(:commit)
    end

    if options[:ref]
      @ref = options.delete(:ref)
    end

    @ref ||= 'master'

    unless options.empty?
      raise ArgumentError, "Unhandled options #{options.keys.inspect} specified for #{self.class}"
    end
  end
end
