
require 'thor'


module Salt
  class Matrix
    class Cli < Thor
      
      autoload :Deploy, 'salt/matrix/cli/deploy'
      
      
      #class_option :trace, :type => :boolean
      
      
      desc 'version', 'Display version information'
      def version
        puts Salt::Matrix::VERSION
      end
      
      desc 'deploy', 'Deploy an environment from Saltfile'
      long_desc <<-LONGDESC
        Process a Saltfile and build an environment from the formulas
        specified within the file. Saltfile consists of one or more entries
        of the form:
        \n
        formula 'NAME', ARG, ARG, ARG ...
        \n
        NAME is the formula name which will be the directory name that
        contains the formula. ARGs will vary depending upon the location
        of the formula. Generally the ARGs will consist of the key :git set
        to the location of the Git repository for the formula and a refernce
        to the Git tag, branch or commit ID. 
        \n
        With the --dir option, one may specify another directory other than
        the current working directory to find the Saltfile.
      LONGDESC
      option :dir, :default => '.'
      option :debug, :default => false
      def deploy 
        if options['debug']
          Salt::Master.set_debug
        end
        deploy_dir = File.expand_path options['dir']
        Salt::Matrix.debug "Deployment directory set to #{deploy_dir}"
        saltfile = Salt::Saltfile.new deploy_dir
        saltfile.load
        saltfile.formulas.each do |f| 
          f.sync
        end
      end
      
      
      desc 'foo', 'Throw an exception'
      def foo
        raise RuntimeError, 'This is foo'
      end
      
      
    end
  end
end