
require 'salt/formula'

module Salt
  
  class Saltfile
    
    attr_reader :basedir
    attr_reader :formuladir
    attr_reader :formulas
    attr_reader :saltfile
    
    # @param [String] basedir
    # @param [String] formuladir
    # @param [String] saltfile
    def initialize(basedir='.', formuladir=nil, saltfile=nil)
      @basedir = basedir
      @formuladir = formuladir || @basedir
      @saltfile = saltfile || File.join(@basedir, 'Saltfile')
      
      @formulas = []
    end
    
    
    def load 
      if File.readable? @saltfile
        self.load!
      else
        $stderr.puts "Saltfile (#{@saltfile}) missing or unreadable"
      end
    end
    
    
    def load!
      dsl = DSL.new(self)
      dsl.instance_eval(saltfile_contents, @saltfile)
    rescue SyntaxError, LoadError => e
      puts e.inspect.magenta
      raise Salt::Matrix::Error.wrap(e, "Failed to process #{@saltfile}")
    end
    
    
    # @param [String] name
    # @param [*Object] args
    def add_formula(name, args)
      @formulas << Salt::Formula.new(name, @formuladir, args)
    end
      
    # @return [File]
    def saltfile_contents
      File.read(@saltfile)
    end
    
    
    
    class DSL
      
      # @param [*Object] librarian
      def initialize(librarian)
        @librarian = librarian
      end
      
      # @param [String] name
      # @param [*Object] args
      def formula(name, args=nil)
        @librarian.add_formula(name, args)
      end
      
      def method_missing(methods, *args)
        raise NoMethodError, "unrecognized declaration: #{method}"
      end
      
    end
    
  end
  
end